/***********************************************************************/
/*                                                                     */
/*                              Pio (C++)                              */
/*                                                                     */
/*                    Polyglot Input/Output Library                    */
/*                                                                     */
/*               EPI Pomdapi, INRIA Paris - Rocquencourt               */
/*                                                                     */
/*  Copyright 2005-2014 INRIA.                                         */
/*  Institut National de Recherche en Informatique et en Automatique.  */
/*  All rights reserved. This file is distributed only by permission.  */
/*                                                                     */
/*  Estelle Marchand <Estelle.Marchand@inria.fr>                       */
/*                                                                     */
/***********************************************************************/

/* $Id: communication.cpp,v 1.9 2014/01/10 19:22:09 fclement Exp $ */

#include "communication.hpp"

namespace io{
  Communication_type Communication::type(){return communication_type ;}
  Communication::Communication(const Phrase& _phrase):phrase(_phrase){communication_type = PHRASE;}
  Communication::Communication(const Task& _task):task(_task){communication_type = TASK;}
  Communication::Communication(const Result& _result):result(_result){communication_type = RESULT;}
  Communication::Communication(const Error& _error):error(_error){communication_type = ERROR;}
  Communication::Communication(const Service& _service):service(_service){communication_type = SERVICE;}
  Phrase & Communication::to_phrase(){return phrase.data();}
  Task & Communication::to_task(){return task.data();}
  Result & Communication::to_result(){return result.data();}
  Error & Communication::to_error(){return error.data();}
  Service & Communication::to_service(){return service.data();}
  
  Communication::Communication(Communication_type ct){
    communication_type = ct;
    switch(ct){
    case PHRASE:
      phrase.Some();break;
    case TASK:
      task.Some();break;
    case RESULT:
      result.Some();break;
    case ERROR:
      error.Some();break;
    case SERVICE:
      service.Some();break;
    default:break;
    }
  }

  Communication::Communication(string s){
    if(s == "%p"){communication_type = PHRASE;
      phrase.Some();
    }
    else  if(s == "%t"){communication_type = TASK;
      task.Some();
    }
    else  if(s == "%r"){communication_type = RESULT;
      result.Some();
    }
    else  if(s == "%e"){communication_type = ERROR;
      error.Some();
    }
    else  if(s == "%s"){communication_type = SERVICE;
      service.Some();
    }
    else{cerr<< "error: invalid communication type: "<<endl;cerr<<s<<endl;cerr<<" given to constructor"<<endl; ElemError
 e; throw(e);}
  }

  string Communication::printable_type(){
    switch(communication_type){
    case PHRASE:
      return "%p";
    case TASK:
      return "%t";
    case RESULT:
      return "%r";
    case ERROR:
      return "%e";
    case SERVICE:
      return "%s";
    default: return "empty";
    }

  }

  Communication::Communication(){communication_type = EMPTYcom;}
  
  Communication::~Communication(){}
  
  unsigned int Communication::size(){
    switch(communication_type){
    case PHRASE: return values().size();
    case TASK: return task.data().n_args();
    case RESULT: return values().size();
    case ERROR: return values().size();
    case SERVICE: return values().size();
    default:{ cerr <<"error: cannot give the size of an empty communication"<<endl;ElemError e;throw(e);}
    }
  }

  
  vector<Typed_value>& Communication::values(){
    switch(communication_type){
    case PHRASE: return phrase.data().values();
    case TASK: return task.data().args();
    case RESULT:return result.data().values();
    case ERROR: return error.data().values();
    case SERVICE: return service.data().values();
    default:{
      cerr <<"error: cannot give the values of an empty communication"<<endl;
      ElemError e; throw(e);}
    }
  }
  
  void Communication::add(Typed_value tv){
    switch(communication_type){
    case PHRASE: phrase.data().add(tv); break;
    case RESULT: result.data().add(tv); break;
    case ERROR: error.data().add(tv); break; 
    case SERVICE: service.data().add(tv); break; 
    case TASK: {cerr <<"error: you cannot add typed_value to a Task communication "<<endl;ElemError e;throw(e);}
    default: {cerr <<"error: cannot add values to an empty communication"<<endl;
    ElemError e;throw(e);}
    } 
  }


}

