(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2013-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: uarray_ng.mli,v 1.2 2014-02-12 17:16:24 fclement Exp $ *)

type index = Upervasives.index;;
type offset = index;;
type length = Upervasives.length;;

val array_get : 'a array -> index -> 'a;;
val array_set : 'a array -> index -> 'a -> unit;;

val array_get_last : 'a array -> 'a;;

val array_sub : 'a array -> offset -> length -> 'a array;;

val array_cum_sub_fold_left_i :
  ('accu -> index -> 'a -> 'accu) ->
  'accu -> 'a array -> offset -> length -> 'accu array
;;

val array_cum_sub_fold_left :
  ('accu -> 'a -> 'accu) ->
  'accu -> 'a array -> offset -> length -> 'accu array
;;

val array_cum_fold_left_i :
  ('accu -> index -> 'a -> 'accu) ->
  'accu -> 'a array -> 'accu array
;;

val array_cum_fold_left :
  ('accu -> 'a -> 'accu) ->
  'accu -> 'a array -> 'accu array
;;

val array_for_all_i : (index -> 'a -> bool) -> 'a array -> bool;;

(** {6 Operations on arrays of arrays} *)

type 'a array_array_t = 'a array array;;

val array_array_map : ('a -> 'b) -> 'a array_array_t -> 'b array_array_t;;
val array_array_map2 :
  ('a -> 'b -> 'c) ->
  'a array_array_t -> 'b array_array_t -> 'c array_array_t
;;

val array_array_pack : 'a array_array_t -> length array * 'a array;;
val array_array_unpack : length array -> 'a array -> 'a array_array_t;;

(** {6 More matrix operations} *)

type line_index = index;;
type column_index = index;;

val matrix_get_line : 'a Umatrix.t -> line_index -> 'a array;;

val matrix_of_array : line_index -> column_index -> 'a array -> 'a Umatrix.t;;

val matrix_map3 :
  ('a -> 'b -> 'c -> 'd) ->
  'a Umatrix.t -> 'b Umatrix.t -> 'c Umatrix.t -> 'd Umatrix.t
;;

(**/**)

val array_unsafe_get : 'a array -> index -> 'a;;
val array_unsafe_set : 'a array -> index -> 'a -> unit;;
